
// Portal-inspired toilet paper holder

// Nudel 2012 - repraprip.blogspot.com

// (C) CC BY-SA 3.0

m3_diameter = 3.6;
m3_nut_diameter = 5.3+0.8;
m3_nut_width = 3;
m3_head_diameter = 7;

rad1=39.25/2+0.4;
rad2=34.5/2-0.25;

lip_width = 4;
lip_size = 1.5;
lip_size2 = lip_size+1;

base_thickness = 1.5+1;

tile_size = [50,50,3+base_thickness];

bevel_size = 2;

rod_r = 19.5/2+2;

width=105; // Width of your roll

extra_width = 14; // Size of rod snap

back_plate_thickness = 4;

back_plate_clip = 0.25; // Extra space to secure clip

snap_walls = 2;
snap_displacement = 2.5;
snap_size = [15,extra_width/2-snap_walls,tile_size.x/2+snap_displacement-5];
snap_slot = 2;
snap_slot_extra = 0.5;

// Base tile
module tile(){
	color([1,1,1])
	difference(){
		translate([0,0,tile_size.z/2]) cube(tile_size,center=true);
		for(i=[0:3]) rotate([0,0,90*i]) translate([0,-tile_size.x/2,tile_size.z]) rotate([45,0,0]) cube([tile_size.x+2,5,bevel_size],center=true);
	}
}

// Tile with slot for portal
module tile_portal(){
	difference(){
		union(){
			// Outer hole
			difference(){
				tile();
				translate([0,0,base_thickness]) cylinder(r=rad1,h=5);
			}
			// Center cylinder
			color([0,1,0]) cylinder(r=rad2,h=3+base_thickness); 
		}
		// Screw hole to attach man and arm
		translate([0,0,-3]) cylinder(r=m3_diameter/2-0.3,h=5,$fn=10);
		// Screw recess
		translate([0,0,-1]) cylinder(r2=m3_diameter/2-0.3,r1=m3_head_diameter/2,h=base_thickness,$fn=10);
		// Arm slot
		translate([0,0,5/2+base_thickness]) cube([5.5,15.5,5],center=true);
	}

	// Snap on tags for portal
	color([1,0,0])
	difference(){
		for(i=[0,1]) rotate ([0,0,180*i]) translate([-lip_width/2,-rad1,5-1]) cube([lip_width,lip_size,1]);
		translate([0,0,5-1.01]) cylinder(r1=rad1,r2=rad1-lip_size2,h=0.51);
		translate([0,0,5-0.5]) cylinder(r1=rad1-lip_size2,r2=rad1,h=0.51);
	}
}

// Module for attaching the center rod to the sides
module rod_snap(variant){
	color([1,1,0])
	translate([-snap_size.x/2*0,-snap_displacement ,0])
	difference(){
		translate([0,8,0]) cylinder(r1=(tile_size.x/2-extra_width/2+5)*0+45/2,r2=rod_r+variant*10,h=extra_width/2-snap_walls);

		// Extra width in opening to make assembly easier
		if(variant==0)  translate([0,15,0]) rotate([-90,0,0]) cylinder(r1=snap_size.x/2,r2=snap_size.x/2+5,h=snap_size.z-6,$fn=6);
		translate([0,0,0])  rotate([-90,0,0]) cylinder(r=snap_size.x/2-variant*0.75,h=tile_size.x,$fn=6);
	}
}

// Center bar
module rod(){
	translate([0,0,-rod_r+rod_r/1.5]) rotate([90,0,0]) 
	difference(){
		translate([0,10,0]) cylinder(r=rod_r,h=width+extra_width);
		rod_snap(1);
		translate([0,0,width+extra_width]) mirror([0,0,1]) rod_snap(1);
		translate([-rod_r,-rod_r/1.5,-1]) cube([rod_r*2,rod_r,width+extra_width+2]);
	}
}

// Aperture Science Logo
module aperture_logo(){
	difference(){
		cylinder(r=20,h=10,center=true);
		cylinder(r=20-9,h=11,center=true);
		for (i=[0:7]) rotate([0,0,(360/8)*i]) translate([20,10,0]) cube([20*2+5,2.5,11],center=true);
	}
}

// Blocks to hold sides and back together
module screw_block(){
	for(i=[-1,1])
	translate([tile_size.x*1.5-6-back_plate_thickness,-tile_size.y/2*i-5+5*i,0]) 
	difference(){
		union(){
			cube([6,10,10]);
			translate([0,5,0]) cylinder(r=5,h=10);
		}
	translate([-4,5,5]) rotate([0,90,0]) rotate([0,0,30]) cylinder(r=m3_diameter/2,h=15);
	translate([5-5,5,5]) rotate([0,90,0]) rotate([0,0,30*0]) cylinder(r=m3_nut_diameter/2,h=m3_nut_width,$fn=6);
	translate([5-5+m3_nut_width-0.01,5,5]) rotate([0,90,0]) rotate([0,0,30*0]) cylinder(r1=m3_nut_diameter/2,r2=m3_nut_diameter/2-0.5,h=1.25,$fn=6);
	translate([5-5,5*i,5-m3_nut_diameter/2]) cube([m3_nut_width,10,m3_nut_diameter]);
	}
}

// Combines other modules to form the sides
module side(mirror){
	mirror([0,0,mirror]){
	rotate([180,0,0]) tile_portal();
	translate([0,0,0]) rod_snap(0);

	difference(){
		rotate([180,0,0]) translate([tile_size.x,0,0]) tile();
		translate([tile_size.x,0,0]) scale([0.6,0.6,1.5]) rotate([0,180*mirror,0]) aperture_logo();
	}
	screw_block();
	}
}

// Back - Needs "portal_back_text.dxf" to render
module back(){

	difference(){
		cube([tile_size.x,width+extra_width,back_plate_thickness]);
		// Corner screw holes
		for(u=[0,1]) for(i=[0,1]) translate([5+(tile_size.x-10)*i,5+(width+extra_width-10)*u,-1]){
			cylinder(r=m3_diameter/2,h=back_plate_thickness+2,$fn=10);
			cylinder(r1=m3_head_diameter/2,r2=m3_diameter/2,h=3,$fn=10);
		}
		// Center screw holes
		for(i=[0,1]) translate([tile_size.x/2,5+(width+extra_width-10)*i,-1]) {
			cylinder(r=m3_diameter/2,h=back_plate_thickness+2,$fn=10);
			translate([0,0,back_plate_thickness-1]) cylinder(r2=m3_head_diameter/2,r1=m3_diameter/2,h=3,$fn=10);		
		}
		// Back text
		translate([tile_size.x,extra_width/2,2]) rotate([0,0,90]) linear_extrude(height=back_plate_thickness+2) import("portal_back_text.dxf");
		// Back bevel
		//translate([-1,(width+extra_width)/2,back_plate_thickness]) rotate([0,45,-90]) cube([2,tile_size.x+2,2]);
		for(i=[0,1]) translate([-1.5+tile_size.x*i,10,back_plate_thickness]) rotate([0,45,0]) cube([2,width+extra_width-20,2]);
	}
	// "Ears"
	for(u=[0,1])
	for(i=[0,1])
		mirror([i,0,0]) translate([-tile_size.x*i+back_plate_clip,(width+extra_width-10)*u,0]) {
		translate([10,10,back_plate_thickness+1.5]) rotate([90,0,0]) 
		intersection(){
			cylinder(r=5,h=10,$fn=20);
			translate([0,0,-1]) cube(12);
		}
		translate([10,0,back_plate_thickness-1]) cube([5,10,1.51+1]);

	}

}

/*
// Test fitting rod + rod_snap
	translate([-rod_r-5,tile_size.y/2,0])
	intersection(){
		rod();
		#translate([-20,-20,-20]) cube([40,40,40]);
	}
	cube([tile_size.x,tile_size.y,2]);
	translate([tile_size.x,tile_size.y,4]/2) rod_snap(0);
*/

/*
// Assembled preview
side(0); // Left 
translate([0,-2+70,0]) rotate([-90,0,0]) rod();
translate([tile_size.x*1.5,tile_size.y/2,0]) rotate([90,0,-90]) back();// Translation to test fit
translate([0,0,width+extra_width]) mirror([0,0,1]) side (0);
*/

translate([-10,-50,-5.5]) rotate([0,0,90]) rod();
side(0); // Left 
translate([0,tile_size.x+5,0]) rotate([180,0,0]) side(1); // Right
translate([tile_size.x*1.5+5,-tile_size.y/2-5,-5.5]) back();